@@ -1,17 +1,25 @@
+//mod dockerreader;
+mod systeminformation;
+use docker_api::image::BuildOpts;
 use std::io::Read;
 use std::path::PathBuf;
 use std::io::Write;
+//use chrono::Utc;
+//use serde_json::json;
 use std::collections::HashMap;
 //use std::time::Duration;
-use std::io::BufReader;
-use std::io::BufRead;
-use std::process::{Command, Stdio};
+//use std::io::BufReader;
+//use std::io::BufRead;
+//use std::process::{Command, Stdio};
 use std::sync::mpsc::{TryRecvError, channel, Receiver};
 use serde::{Deserialize, Serialize};
 use regex::Regex;
 use std::fs;
 use structopt::StructOpt;
-
+//use futures_util::future::ready;
+use futures_util::stream::StreamExt;
+//use bollard::{Docker, image::BuildImageOptions, models::HostConfig, container::{CreateContainerOptions, Config, LogsOptions, LogOutput}};
+use docker_api::{Docker, api::{ContainerCreateOpts, image::models::ImageBuildChunk}};
 /// Builds and runs benchmarks from https://youtu.be/D3h62rgewZM
 #[derive(StructOpt, Debug, Clone)]
 #[structopt(name = "plummerprimes")]
@@ -24,8 +32,8 @@ struct PlummerPrimesConfig {
     #[structopt(short, long, default_value="table")]
     formatter: String,
 
-    /// Write save to given file
-    #[structopt(short, long="save-file", parse(from_os_str), default_value="save.db")]
+    /// Read/Write save from/to given file
+    #[structopt(long="save-file", parse(from_os_str), default_value="save.db")]
     save_file: PathBuf,
 
     /// Write report file(s) to given file
@@ -52,15 +60,44 @@ struct PlummerPrimesConfig {
     #[structopt(short, long="list-formatters")]
     list_formatters: bool,
 
+    /// Build and run specified solution name [example: primerust_solution_1]
+    #[structopt(short, long)]
+    solution: Option<String>,
+
+    /// Make json output compatible with the primes project's nodejs output
+    #[structopt(long)]
+    compat: bool,
 }
 
-#[derive(Deserialize, Clone, Debug)]
+
+#[derive(Serialize, Deserialize, Clone, Debug)]
 struct RegexOutput {
     label: String,
     passes: usize,
     duration: String,
     threads: usize,
-    extra: String,
+    extra: HashMap<String, String>,
+}
+
+impl RegexOutput {
+    fn new(reg: Regex, extra_reg: Regex, input: &str) -> RegexOutput {
+        let caps = reg.captures(input).unwrap();
+        let mut extra: HashMap<String, String> = HashMap::new();
+        if let Some(x) = caps.name("extra") {
+            if extra_reg.is_match(x.as_str()) {
+                for caps_extra in extra_reg.captures_iter(x.as_str()) {
+                    extra.insert(caps_extra.name("key").unwrap().as_str().to_string(), caps_extra.name("value").unwrap().as_str().to_string());
+                }
+            }
+        }
+        Self {
+           label: caps.name("label").unwrap().as_str().to_string(),
+           passes: caps.name("passes").unwrap().as_str().parse::<usize>().unwrap(),
+           duration: caps.name("duration").unwrap().as_str().to_string(),
+           threads: caps.name("threads").unwrap().as_str().parse::<usize>().unwrap(),
+           extra: extra,
+        }
+    }
 }
 
 #[derive(Clone, Deserialize, Serialize, Debug)]
@@ -75,6 +112,7 @@ struct RunOutput {
 
 #[derive(Clone, Deserialize, Serialize, Debug)]
 struct SaveOutput {
+    extra_data: crate::systeminformation::ExtraData,
     single: Vec<HashMap<String, Option<String>>>,
     multi: Vec<HashMap<String, Option<String>>>,
     broken_solutions: Vec<String>,
@@ -87,47 +125,157 @@ struct LangOutput {
     solution_name: String,
 }
 
+#[derive(Clone, Deserialize, Serialize, Debug)]
+struct CompatOutput {
+    implementation: String,
+    solution: String,
+    label: String,
+    passes: usize,
+    duration: f64,
+    threads: usize,
+    tags: HashMap<String, String>,
+}
+
+//impl From<SaveOutput> for CompatOutput {
+//    fn from(val: SaveOutput) -> CompatOutput {
+//        let implementation = 
+//    }
+//} 
+
 impl From<RegexOutput> for RunOutput {
     fn from(val: RegexOutput) -> RunOutput {
+        /*
         let mut extra: HashMap<String, String> = HashMap::new();
-        for key_val in val.extra.split(",") {
-            let (key, value) = key_val.split_once("=").unwrap();
-            extra.insert(key.to_string(), value.to_string());
+        if let Some(val_extra) = val.clone().extra {
+            for key_val in val_extra.split(",") {
+                let (key, value) = key_val.split_once("=").unwrap();
+                extra.insert(key.to_string(), value.to_string());
+            }
         }
+        */
         RunOutput {
             label: val.clone().label,
             passes: val.clone().passes,
             duration: val.clone().duration,
             threads: val.threads,
             pps: (val.passes as f64 / val.duration.parse::<f64>().unwrap()),
-            extra: extra,
+            extra: val.extra,
         }
     }
 }
-pub fn run_benchmarks(rawregex: &str, target: &str) {
+
+
+#[derive(Clone, Deserialize, Serialize, Debug)]
+struct JSONCompatOutput {
+    version: String,
+    metadata: crate::systeminformation::MetaData,
+    machine: crate::systeminformation::Machine,
+    results: Vec<CompatOutput>,
+}
+/*
+impl JSONCompatOutput {
+    async fn new(results: Vec<CompatOutput>, extra_data: crate::systeminformation::ExtraData) -> Self {
+        Self {
+            version: extra_data.version,
+            metadata: extra_data.metadata,
+            machine: extra_data.machine,
+            results: results,
+        }
+    }
+}
+*/
+
+impl From<SaveOutput> for JSONCompatOutput {
+    fn from(val: SaveOutput) -> JSONCompatOutput {
+        let extra_data = val.clone().extra_data;
+        let mut all_results = val.clone().single;
+        all_results.append(&mut val.multi.clone());
+        let mut new_results: Vec<CompatOutput> = Vec::new();
+        for old_result in all_results {
+            let mut tags: HashMap<String, String> = HashMap::new();
+            let mut mut_result = old_result.clone();
+            let implementation = mut_result["language"].clone().unwrap();
+            mut_result.remove("language");
+            let solution_name = mut_result["solution_name"].clone().unwrap();
+            let solution = solution_name.split("_").last().unwrap();
+            mut_result.remove("solution_name");
+            let label = mut_result["label"].clone().unwrap();
+            mut_result.remove("label");
+            let threads = mut_result["threads"].clone().unwrap().parse::<usize>().unwrap();
+            mut_result.remove("threads");
+            let duration = mut_result["duration"].clone().unwrap().parse::<f64>().unwrap();
+            mut_result.remove("duration");
+            let passes = mut_result["passes"].clone().unwrap().parse::<usize>().unwrap();
+            mut_result.remove("passes");
+            for tag_name in mut_result.clone().keys() {
+                let res = mut_result[tag_name].clone();
+                if res.is_none() {
+                    mut_result.remove(tag_name);
+                } else {
+                    tags.insert(tag_name.to_string(), res.unwrap().to_string());
+                }
+            }
+            new_results.push(CompatOutput {
+                duration: duration,
+                implementation: implementation.to_string(),
+                label: label.to_string(),
+                passes: passes,
+                solution: solution.to_string(),
+                threads: threads,
+                tags: tags,
+            });
+        }
+        JSONCompatOutput {
+            version: extra_data.version,
+            metadata: extra_data.metadata,
+            machine: extra_data.machine,
+            results: new_results,
+        }
+    }
+}
+
+pub async fn run_benchmarks(raw_regex: &str, extra_regex: &str, target: &str) {
+    //let mut sys_info = smbioslib::table_load_from_device().unwrap();
+    //let base_info: Vec<smbioslib::SMBiosProcessorInformation> = sys_info.collect();
+    //for info in base_info {
+    //    println!("{:?}", info.processor_);
+    //}
+    //return;
+    let arch = target.split_once("-").unwrap().0;
+    let arch_file: &str;
+    let nodejs_arch: String;
+    if arch == "x86_64" {
+        arch_file = "arch-amd64";
+        nodejs_arch = "x64".to_string();
+    } else if arch == "aarch64" {
+        arch_file = "arch-arm64";
+        nodejs_arch = "arm64".to_string();
+    } else {
+        panic!("Unsupported architecture!");
+    }
+    let extra_data = crate::systeminformation::ExtraData::new(nodejs_arch).await;
+    //println!("{}", serde_json::to_string_pretty(&extra_data).unwrap());
+    //return;
     let opts = PlummerPrimesConfig::from_args();
     //let input_file = opts.clone().save_file;
     //let formatter = opts.clone().formatter;
     //let output_file = opts.clone().report_file;
     if opts.clone().list_formatters {
-        println!("List of formatters: table, html, csv");
+        println!("List of formatters: table, html, csv, json");
         return;
     }
     if opts.only_output {
-        output_report(opts.clone().formatter, opts.clone().save_file, opts.clone().report_base, opts.clone().report_dir);
+        let mut f = fs::File::open(opts.clone().save_file).unwrap();
+        let mut buf: Vec<u8> = Vec::new();
+        f.read_to_end(&mut buf).unwrap();
+        output_report(opts.clone().formatter, buf, opts.clone().report_base, opts.clone().report_dir, opts.clone().compat);
         return;
     }
-    let arch = target.split_once("-").unwrap().0;
-    let arch_file: &str;
+    //let arch_file: &str;
     let mut broken_solutions: Vec<String> = Vec::new();
-    if arch == "x86_64" {
-        arch_file = "arch-amd64";
-    } else if arch == "aarch64" {
-        arch_file = "arch-arm64";
-    } else {
-        panic!("Unsupported architecture!");
-    }
-    let reg = Regex::new(rawregex).unwrap();
+    
+    let reg = Regex::new(raw_regex).unwrap();
+    let extra_reg = Regex::new(extra_regex).unwrap();
     let dir_root = std::env::current_dir().unwrap();
     //let dir_root = cur_dir.parent().unwrap();
     let (tx, rx) = channel::<String>();
@@ -156,6 +304,11 @@ pub fn run_benchmarks(rawregex: &str, target: &str) {
             let num_solutions = fs::read_dir(local_dir.clone()).unwrap().filter(|x| x.as_ref().unwrap().file_name().into_string().unwrap().starts_with("solution_")).count();
             for solution_num in 1..(num_solutions + 1) {
                 let name = format!("prime{}_solution_{}", lang.to_lowercase(), solution_num);
+                if let Some(solution_name) = opts.clone().solution {
+                    if name.clone() != solution_name {
+                        continue;
+                    }
+                }
                 let solution_path = local_dir.clone().join(format!("solution_{}", solution_num).as_str());
                 if fs::read_dir(solution_path.clone()).unwrap().filter(|x| x.as_ref().unwrap().file_name().into_string().unwrap().starts_with("arch-")).count() > 0 {
                     if !solution_path.clone().join(arch_file).exists() {
@@ -167,11 +320,11 @@ pub fn run_benchmarks(rawregex: &str, target: &str) {
                     eprintln!("Skipping {} because build is disabled", name.clone());
                     continue;
                 }
-                let indiv_sols = get_solutions_from(reg.clone(), name.clone(), &rx, opts.clone(), solution_path.clone());
+                let indiv_sols = get_solutions_from(reg.clone(), extra_reg.clone(), name.clone(), &rx, opts.clone(), solution_path.clone()).await;
                 for sol in indiv_sols.clone() {
                     benchmarks.push(LangOutput {language: lang.clone(), output: sol, solution_name: name.clone()});
                     if opts.debug {
-                        println!("{}", benchmarks.clone().len());
+                        eprintln!("{}", benchmarks.clone().len());
                     }
                 }
                 if indiv_sols.clone().len() == 0 {
@@ -242,29 +395,78 @@ pub fn run_benchmarks(rawregex: &str, target: &str) {
         }
     }
     if opts.debug {
-        println!("Multi: {:?}", save_output_multi);
-        println!("Single: {:?}", save_output_single);
-        println!("Broken Solutions: {:?}", broken_solutions);
+        eprintln!("Multi: {:?}", save_output_multi);
+        eprintln!("Single: {:?}", save_output_single);
+        eprintln!("Broken Solutions: {:?}", broken_solutions);
     }
     let save_output = SaveOutput {
+        extra_data: extra_data,
         single: save_output_single,
         multi: save_output_multi,
         broken_solutions: broken_solutions,
     };
-    let mut f = fs::File::create(opts.clone().save_file).unwrap();
-    f.write_all(&rmp_serde::encode::to_vec(&save_output).unwrap()).unwrap();
-    println!("Saved data to {}", opts.clone().save_file.to_str().unwrap());
-    //output_report(opts.clone().formatter, opts.clone().save_file, opts.clone().report_base, );
-    output_report(opts.clone().formatter, opts.clone().save_file, opts.clone().report_base, opts.clone().report_dir);
+    if opts.clone().solution.is_none() {
+        let mut f = fs::File::create(opts.clone().save_file).unwrap();
+        f.write_all(&rmp_serde::encode::to_vec(&save_output).unwrap()).unwrap();
+        println!("Saved data to {}", opts.clone().save_file.to_str().unwrap());
+    }
+    output_report(opts.clone().formatter, rmp_serde::encode::to_vec(&save_output).unwrap(), opts.clone().report_base, opts.clone().report_dir, opts.clone().compat);
     //println!("Saved report to {}", opts.clone().report_file.to_str().unwrap());
 }
-fn get_solutions_from(reg: Regex, name: String, rx: &Receiver<String>, opts: PlummerPrimesConfig, directory: PathBuf) -> Vec<RunOutput> {
+async fn process_docker_output(message: &[u8], buffer: String, outs: Vec<RunOutput>, rx: &Receiver<String>, reg: Regex, extra_reg: Regex,  opts: PlummerPrimesConfig) -> (String, Vec<RunOutput>, String) {
+    let mut buf = buffer.clone();
+    let mut outputs = outs.clone();
+    //let mut had_error = false;
+    //let mut status = prev_status.clone();
+    let mut status = String::new();
+    let mut last = String::new();  
+    buf.push_str(String::from_utf8_lossy(message).into_owned().as_str());
+    let split: Vec<&str> = buf.split("\n").collect();
+    let num_items = split.len();
+    for (num, b) in split.clone().into_iter().enumerate() {
+        last = b.to_string();
+        status = match rx.try_recv() {
+            Ok(rx) => rx,
+            Err(TryRecvError::Empty) => "keepgoing".to_string(),
+            Err(TryRecvError::Disconnected) => "disconnected".to_string(), 
+        };
+        if status.as_str() != "keepgoing" {
+            std::process::exit(1);
+        }
+        if num == num_items - 1 && last == String::new() {
+            break;
+        }
+        if opts.debug && last != String::new() {
+            eprintln!("{}", last);
+        }
+        if reg.clone().is_match(last.as_str()) {
+            if opts.debug {
+                eprintln!("Matched");
+            }
+            //let reg_out: RegexOutput = de_regex::from_str_regex(last.as_str(), reg.clone()).unwrap();
+            let reg_out = RegexOutput::new(reg.clone(), extra_reg.clone(), last.as_str());
+            let run_out: RunOutput = reg_out.into();
+            outputs.push(run_out);
+        } else if opts.debug {
+            eprintln!("Not a match");
+        }
+    }
+
+    (last, outputs, status)
+}
+async fn get_solutions_from(reg: Regex, extra_reg: Regex, name: String, rx: &Receiver<String>, opts: PlummerPrimesConfig, directory: PathBuf) -> Vec<RunOutput> {
+    let mut status = match rx.try_recv() {
+        Ok(rx) => rx,
+        Err(TryRecvError::Empty) => "keepgoing".to_string(),
+        Err(TryRecvError::Disconnected) => "disconnected".to_string(), 
+    };
     let mut base_run_args = "run --rm ".to_string();
     if opts.unconfined {
         base_run_args.push_str("--security-opt seccomp=unconfined ")
     }
     base_run_args.push_str(name.as_str());
     let mut skip_this = false;
+    /*
     {
         println!("Building docker container for {}...", name.clone());
         let mut child = Command::new("docker").arg("build").arg("-t").arg(name.as_str()).arg(directory)
@@ -281,7 +483,6 @@ fn get_solutions_from(reg: Regex, name: String, rx: &Receiver<String>, opts: Plu
         println!("Error building docker container...\nSkipping run...");
         return outputs;
     }
-    println!("Running {}...", name);
     let mut cmd = Command::new("docker");
     let mut c = cmd.args(base_run_args.split(" "));
     if !opts.debug {
@@ -291,12 +492,86 @@ fn get_solutions_from(reg: Regex, name: String, rx: &Receiver<String>, opts: Plu
         .stdout(Stdio::piped())
         .spawn().unwrap();
     let mut reader = BufReader::new(child.stdout.take().unwrap());
-    let mut status = match rx.try_recv() {
-        Ok(rx) => rx,
-        Err(TryRecvError::Empty) => "keepgoing".to_string(),
-        Err(TryRecvError::Disconnected) => "disconnected".to_string(), 
-    };
+    */
+    
+    println!("Building docker container for {}...", name.clone());
+    let docker = Docker::new("unix:///var/run/docker.sock").unwrap();
+    let options = BuildOpts::builder(directory).tag(name.clone()).build();
+    let mut stream = docker.images().build(&options);
+    while let Some(build_result) = stream.next().await {
+        status = match rx.try_recv() {
+            Ok(rx) => rx,
+            Err(TryRecvError::Empty) => "keepgoing".to_string(),
+            Err(TryRecvError::Disconnected) => "disconnected".to_string(), 
+        };
+        if status.as_str() != "keepgoing" {
+            std::process::exit(1);
+        }
+        match build_result {
+            Ok(output) => {
+                if opts.debug {
+                    eprintln!("[BUILD] {:?}", output);
+                }
+                if let ImageBuildChunk::Update { stream } = output.clone() {
+                    if stream.starts_with(format!("Successfully tagged {}", name.clone()).as_str()) {
+                        break;
+                    }
+                }
+                if let ImageBuildChunk::Error { error, error_detail } = output.clone() {
+                    eprintln!("Error: {:?} {:?}", error, error_detail);
+                }
+            },
+            Err(e) => {
+                eprintln!("Error: {}", e);
+                skip_this = true;
+                break;
+            },
+        }
+    }
+    let mut outputs: Vec<RunOutput> = Vec::new();
+    if skip_this {
+        eprintln!("Error building docker container...\nSkipping run...");
+        return outputs;
+    }
+    println!("Running {}...", name);
+
+    let mut container_opts = ContainerCreateOpts::builder(name.as_str()).name(format!("{}_container", name).as_str()).auto_remove(true);
+    //container_opts.params.insert("HostConfig.SecurityOpt", json!(vec!["seccomp=unconfined"]));
+    if opts.clone().unconfined {
+        container_opts = container_opts.security_options(vec!["seccomp=unconfined".to_string()])
+    }
+    //.build();
+    let container = docker.containers().create(&container_opts.build()).await.unwrap();
+    let start_result = container.start().await;
+    if let Err(e) = start_result {
+        if let docker_api::errors::Error::Fault { code: _code, message } = e {
+            eprintln!("Error: {}", message);
+        } else {
+            eprintln!("Error: {:?}", e);
+        }
+        return outputs;
+    }
+    //let logs_options = LogsOpts::builder().stdout(true).stderr(opts.clone().debug).build();
+    //let logs = container.logs(&logs_options).await
+    let tty = container.attach().await.unwrap();
+    let mut stream = tty.split().0;
+
+    //let mut had_error = true;
+    let mut buf = String::new();
+    //let mut logs = container.logs(&logs_options);
+    while let Some(msg) = stream.next().await {
+        if status.as_str() != "keepgoing" {
+            break
+        }
+        //println!("Working...");
+        let message = msg.unwrap();
+        let x = process_docker_output(message.as_ref(), buf, outputs, rx, reg.clone(), extra_reg.clone(), opts.clone()).await;
+        buf = x.0.clone();
+        outputs = x.1.clone();
+        status = x.2.clone();
+    }
     //let mut buff_status = "keepgoing";
+    /*
     let mut line = String::new();
     let mut buff_status = match reader.read_line(&mut line) {
         Ok(0) => "done",
@@ -309,7 +584,8 @@ fn get_solutions_from(reg: Regex, name: String, rx: &Receiver<String>, opts: Plu
             println!("'{}'", line);
         }
         if reg.clone().is_match(line.as_str()) {
-            let reg_out: RegexOutput = de_regex::from_str_regex(line.as_str(), reg.clone()).unwrap();
+            let reg_out = RegexOutput::new(reg.clone(), last.as_str());
+            //let reg_out: RegexOutput = de_regex::from_str_regex(line.as_str(), reg.clone()).unwrap();
             let run_out: RunOutput = reg_out.into();
             outputs.push(run_out);
         } else if opts.debug {
@@ -338,21 +614,32 @@ fn get_solutions_from(reg: Regex, name: String, rx: &Receiver<String>, opts: Plu
         child.wait().unwrap();
         std::process::exit(1);
     }
+    */
     //assert_eq!(true, outputs.len() > 0 || vec!["primeawk_solution_1", "primecomal_solution_1", "primecsharp_solution_2"].into_iter().map(|a| a.to_string()).collect::<Vec<String>>().contains(&name));
     outputs
 }
-fn output_report(formatter: String, save_file: PathBuf, base_name: String, report_dir: PathBuf) {
-    let mut f = fs::File::open(save_file).unwrap();
-    let mut buf: Vec<u8> = Vec::new();
-    f.read_to_end(&mut buf).unwrap();
+fn output_report(formatter: String, buf: Vec<u8>, base_name: String, report_dir: PathBuf, compat: bool) {
+    //let mut f = fs::File::open(save_file).unwrap();
+    //let mut buf: Vec<u8> = Vec::new();
+    //f.read_to_end(&mut buf).unwrap();
     let save_output: SaveOutput = rmp_serde::decode::from_slice(&buf).unwrap();
-    let mut alpha_single_keys = save_output.single[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
-    let mut alpha_multi_keys = save_output.multi[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
+    let save_output_clone = save_output.clone();
+    let mut alpha_single_keys: Vec<String> = Vec::new();
+    let mut alpha_multi_keys: Vec<String> = Vec::new();
+    let mut output_multi = false;
+    let mut output_single = false;
+    if save_output.single.len() > 0 {
+        alpha_single_keys = save_output.single[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
+        output_single = true;
+    }
+    if save_output.multi.len() > 0 {
+        alpha_multi_keys = save_output.multi[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
+        output_multi = true;
+    }
     alpha_single_keys.sort();
     alpha_multi_keys.sort();
     let mut table_single = table_print::Table::new(alpha_single_keys.clone());
     let mut table_multi = table_print::Table::new(alpha_multi_keys.clone());
-
     //let mut vec_single: Vec<Vec<String>> = Vec::new();
     //let mut vec_multi: Vec<Vec<String>> = Vec::new();
     for out in save_output.single {
@@ -381,10 +668,16 @@ fn output_report(formatter: String, save_file: PathBuf, base_name: String, repor
         table_multi.insert_row(this_vec);
     }
     if formatter.as_str() == "html" {
-        let mut html_data = "<html>\n<head>\n<title>PlummerPrimes Output</title>\n</head>\n<body>\n<p>Single Threaded</p>\n".to_string();
-        html_data.push_str(table_single.get_html_element().as_str());
-        html_data.push_str("\n<p>Multi Threaded</p>\n");
-        html_data.push_str(table_multi.get_html_element().as_str());
+        let mut html_data = "<html>\n<head>\n<title>PlummerPrimes Output</title>\n</head>\n<body>\n".to_string();
+        if output_single {
+            html_data.push_str("<p>Single Threaded</p>\n");
+            html_data.push_str(table_single.get_html_element().as_str());
+        }
+        if output_multi {
+            html_data.push_str("\n<p>Multi Threaded</p>\n");
+            html_data.push_str(table_multi.get_html_element().as_str());
+        }
+        html_data.push_str("\n</body>\n</html>");
         let output_path = report_dir.join(format!("{}.html", base_name).as_str());
         let mut o = fs::File::create(output_path.clone()).unwrap();
         o.write_all(html_data.as_bytes()).unwrap();
@@ -392,75 +685,56 @@ fn output_report(formatter: String, save_file: PathBuf, base_name: String, repor
     } else if formatter.as_str() == "csv" {
         let output_path_single = report_dir.join(format!("{}_single.csv", base_name).as_str());
         let output_path_multi = report_dir.join(format!("{}_multi.csv", base_name).as_str());
-        let mut o = fs::File::create(output_path_single.clone()).unwrap();
-        o.write_all(table_single.get_csv().as_bytes()).unwrap();
-        drop(o);
-        println!("Saved report to {}", output_path_single.to_str().unwrap());
-        let mut o = fs::File::create(output_path_multi.clone()).unwrap();
-        println!("Saved report to {}", output_path_multi.to_str().unwrap());
-        o.write_all(table_multi.get_csv().as_bytes()).unwrap();
+        if output_single {
+            let mut o = fs::File::create(output_path_single.clone()).unwrap();
+            o.write_all(table_single.get_csv().as_bytes()).unwrap();
+            println!("Saved report to {}", output_path_single.to_str().unwrap());
+        }
+        if output_multi {
+            let mut o = fs::File::create(output_path_multi.clone()).unwrap();
+            println!("Saved report to {}", output_path_multi.to_str().unwrap());
+            o.write_all(table_multi.get_csv().as_bytes()).unwrap();
+        }
     } else if formatter.as_str() == "table" {
         let width = termsize::get().unwrap().cols;
-        println!("Single-threaded: ");
-        match table_single.get_pretty(width.into()) {
-            Ok(s) => {
-                println!("{}", s);
-            },
-            Err(_) => {
-                println!("Terminal not wide enough to display table.\nTry using --formatter with a different formatter than table.\nYou can use the --list-formatters option to get a list.");
+        
+        if output_single {
+            println!("Single-threaded: ");
+            match table_single.get_pretty(width.into()) {
+                Ok(s) => {
+                    println!("{}", s);
+                },
+                Err(_) => {
+                    println!("Terminal not wide enough to display table.\nTry using --formatter with a different formatter than table.\nYou can use the --list-formatters option to get a list.");
+                }
             }
         }
-        println!("Multi-threaded: ");
-        match table_multi.get_pretty(width.into()) {
-            Ok(s) => {
-                println!("{}", s);
-            },
-            Err(_) => {
-                println!("Terminal not wide enough to display table.\nTry using --formatter with a different formatter than table.\nYou can use the --list-formatters option to get a list.");
+        if output_multi {
+            println!("Multi-threaded: ");
+            match table_multi.get_pretty(width.into()) {
+                Ok(s) => {
+                    println!("{}", s);
+                },
+                Err(_) => {
+                    println!("Terminal not wide enough to display table.\nTry using --formatter with a different formatter than table.\nYou can use the --list-formatters option to get a list.");
+                }
             }
         }
         //println!("{}", table_single)
-    }
-    /*
-    if formatter == "html" {
-        let mut html_data = "<html>\n<head><title>plummerprimes output</title></head>\n<body>\n<p>Single threaded</p>\n<table>\n<tr>\n<th>".to_string();
-        let mut alpha_single_keys = save_output.single[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
-        let mut alpha_multi_keys = save_output.multi[0].keys().map(|b| b.to_string()).collect::<Vec<String>>();
-        alpha_single_keys.sort();
-        alpha_multi_keys.sort();
-        html_data.push_str(alpha_single_keys.clone().join("</th>\n<th>").as_str());
-        html_data.push_str("</th>\n</tr>\n<tr>\n");
-        for out in save_output.single {
-            for key in alpha_single_keys.clone() {
-                let mut value = String::new();
-                let val = out.get(&key).unwrap();
-                if val.is_some() {
-                    value = val.as_ref().unwrap().clone();
-                } 
-                html_data.push_str(format!("<td>{}</td>", value).as_str());
-                html_data.push_str("\n");
-            }
-            html_data.push_str("</tr>\n");
+    } else if formatter.as_str() == "json" {
+        let json_data: String;
+        let mut output_path = report_dir.clone();
+        if compat {
+            output_path = output_path.join(format!("{}_compat.json", base_name).as_str());
+            let compat_json: JSONCompatOutput = save_output_clone.into();
+            json_data = serde_json::to_string_pretty(&compat_json).unwrap();
+        } else {
+            output_path = output_path.join(format!("{}.json", base_name).as_str());
+            json_data = serde_json::to_string_pretty(&save_output_clone).unwrap();
         }
-        html_data.push_str("</table>\n<p>Multi threaded</p>\n<table>\n<tr>\n<th>");
-        html_data.push_str(alpha_multi_keys.clone().join("</th>\n<th>").as_str());
-        html_data.push_str("</th>\n</tr>\n<tr>\n");
-        for out in save_output.multi {
-            for key in alpha_multi_keys.clone() {
-                let mut value = String::new();
-                let val = out.get(&key).unwrap();
-                if val.is_some() {
-                    value = val.as_ref().unwrap().clone();
-                } 
-                html_data.push_str(format!("<td>{}</td>", value).as_str());
-                html_data.push_str("\n");
-            }
-            html_data.push_str("</tr>\n");
-        }
-        html_data.push_str("</table>\n</body>\n</html>");
-        let mut o = fs::File::create(output_file).unwrap();
-        o.write_all(html_data.as_bytes()).unwrap();
+        let mut o = fs::File::create(output_path.clone()).unwrap();
+        o.write_all(json_data.as_bytes()).unwrap();
+        println!("Saved report to {}", output_path.to_str().unwrap());
     }
-    */
-
 }
+
